/*
	Code generation for the following grammar segment:
	
	BehaviorDeclaration := "behavior" <IDENTIFIER> ["extends" Name()] ["implements" Name() ("," Name())*] BehaviorBody()
*/


package salsa_lite.core.compiler.definitions;

import java.util.Vector;

import salsa_lite.core.compiler.SalsaCompiler;
import salsa_lite.core.compiler.SalsaParser;
import salsa_lite.core.compiler.SimpleNode;

public class BehaviorDeclaration extends SimpleNode {

	public BehaviorDeclaration(int id) 			{ super(id); }
	public BehaviorDeclaration(SalsaParser p, int id)	{ super(p, id); }

	public String extendsName = null;
	public Vector implementNames = new Vector();

	public String getActorName() {
		return getToken(1).image;
	}

	public String getReferenceCode() {
		String code = "class " + getToken(1);					//"behavior <IDENTIFIER>"

		if (tokens.length == 2) {
			String actorType = System.getProperty("actor");
			if (actorType == null) {
				if (System.getProperty("enable_gc") == null) {
					extendsName = "LiteActor";
					code += " extends LiteActor ";
				} else {
					extendsName = "CollectibleActor";
					code += " extends CollectibleActor ";
				}
			} else {
				extendsName = actorType;
				code += " extends " + actorType + " ";
			}
		}

		int i = 2;
		for (; i < tokens.length; i++) {
			if ( getToken(i).image.equals("extends") ) {
				extendsName = getChild(i-2).getJavaCode();
				code += " " + getToken(i).image + " " + getChild(i-2).getJavaCode();
			} else if ( getToken(i).image.equals(",") ) {
				implementNames.add(getChild(i-2).getJavaCode());
				code += ", " + getChild(i-2).getJavaCode();
			} else if ( getToken(i).image.equals("implements") ) {
 				code += " extends ";
				code += getChild(i-2).getJavaCode();
			}
		}
		SalsaCompiler.extendsName = extendsName;
		SalsaCompiler.implementNames = implementNames;
		SalsaCompiler.actorName = getToken(1).image;

		return code + " ";
	}

	public String getChildCode() {
		String code = "class " + getToken(1) + "State";					//"behavior <IDENTIFIER>"

		if (tokens.length == 2 || !getToken(2).image.equals("extends")) {
			String actorType = System.getProperty("actor");
			if (actorType == null) {
				if (System.getProperty("enable_gc") == null) {
					extendsName = "LiteActorState";
					code += " extends LiteActorState ";
				} else {
					extendsName = "CollectibleActorState";
					code += " extends CollectibleActorState ";
				}
			} else {
				extendsName = actorType + "State";
				code += " extends " + actorType + "State ";
			}
		}

		int i = 2;
		for (; i < tokens.length; i++) {
			if ( getToken(i).image.equals("extends") ) {
				extendsName = getChild(i-2).getJavaCode() + "State";
				code += " " + getToken(i).image + " " + getChild(i-2).getJavaCode() + "State";
			} else if ( getToken(i).image.equals(",") ) {
				implementNames.add(getChild(i-2).getJavaCode() + "State");
				code += ", " + getChild(i-2).getJavaCode() + "State";
			} else if ( getToken(i).image.equals("implements") ) {
				code += "implements ";
				code += getChild(i-2).getJavaCode() + "State";
			}
		}
		SalsaCompiler.extendsName = extendsName;
		SalsaCompiler.implementNames = implementNames;
		SalsaCompiler.actorName = getToken(1).image + "State";


		code += " " + getChild(i-2).getJavaCode();

		return code;
	}
}
