/*
	Code generation for the following grammar segment:

	CompilationUnit := 
		[ModuleDeclaration()] 
		(ImportDeclaration())* 
		[BehaviorDeclarationAttributes()]
		( BehaviorDeclaration() | InterfaceDeclaration() )
 */

package salsa_lite.core.compiler.definitions;

import salsa_lite.core.compiler.SalsaCompiler;
import salsa_lite.core.compiler.SalsaParser;
import salsa_lite.core.compiler.SimpleNode;

public class CompilationUnit extends SimpleNode {
	public CompilationUnit(int id) 			{ super(id); }
	public CompilationUnit(SalsaParser p, int id)	{ super(p, id); }

	public String getModule() {
		if (getChild(0) instanceof ModuleDeclaration) return ((ModuleDeclaration)getChild(0)).getModule();
		else return "";
	}

	public String getReferenceCode() {
		int place = 0;

		String code = "";
		if (getChild(0) instanceof ModuleDeclaration) {
			code += getChild(0).getJavaCode();
			place++;
		}

		code += "// Import declarations generated by the SALSA compiler, do not modify.\n";

		code += "import salsa_lite.core.language.Actor;\n";
		code += "import salsa_lite.core.language.ActorReference;\n";
		code += "import salsa_lite.core.language.IdentifierGenerator;\n";
		code += "import salsa_lite.core.language.Message;\n";
		code += "import salsa_lite.core.services.ServiceFactory;\n";


		boolean isAbstract = !(getChild(children.length-1) instanceof BehaviorDeclaration);

		String actorType = System.getProperty("actor");
		if (actorType == null) {
			if (System.getProperty("enable_gc") == null) {
				actorType = "salsa_lite.core.language.LiteActor";
			} else {
				actorType = "salsa_lite.core.language.CollectibleActor";
			}
		}

		code += "\n";
		code += "import " + actorType + ";\n\n";
		code += "import salsa_lite.core.naming.UAN;\n";
		code += "import salsa_lite.core.naming.UAL;\n";
		code += "import salsa_lite.core.naming.MalformedUALException;\n";
		code += "import salsa_lite.core.naming.MalformedUANException;\n\n";
		code += "// End SALSA compiler generated import delcarations.\n\n";

		for (int i = place; i < (children.length-1); i++) {
			if (getChild(i) instanceof BehaviorDeclarationAttributes) code += "\n";
			code += getChild(i).getJavaCode();
		}

		String actorName;
		if (getChild(children.length-1) instanceof BehaviorDeclaration) {
			code += ((BehaviorDeclaration)getChild(children.length-1)).getReferenceCode() + "{\n";
			actorName = ((BehaviorDeclaration)getChild(children.length-1)).getActorName();
		} else {
			code += ((InterfaceDeclaration)getChild(children.length-1)).getReferenceCode() + "{\n";
			actorName = ((InterfaceDeclaration)getChild(children.length-1)).getActorName();
		}

		SalsaCompiler.indent++;
		/********
			*       Create constructors which will create references.
		 ********/
		code += SalsaCompiler.getIndent() + "public " + actorName + "(UAN uan) { super(uan); }\n";
		code += SalsaCompiler.getIndent() + "public " + actorName + "(UAL ual) { super(ual); }\n";

		if (System.getProperty("enable_gc") != null) {
			code += SalsaCompiler.getIndent() + "public " + actorName + " copy() {\n";
			code += SalsaCompiler.getIndent() + "\t" + actorName + " copy;\n";
			code += SalsaCompiler.getIndent() + "\tif (getUAN() != null) {\n";
			code += SalsaCompiler.getIndent() + "\t\tcopy = new " + actorName + "(getUAN());\n";
			code += SalsaCompiler.getIndent() + "\t} else {\n";
			code += SalsaCompiler.getIndent() + "\t\tcopy = new " + actorName + "(getUAL());\n";
			code += SalsaCompiler.getIndent() + "\t}\n";
			code += SalsaCompiler.getIndent() + "\tcopy.selfReference = false;\n";
			code += SalsaCompiler.getIndent() + "\tServiceFactory.getGarbage().incrementReference(getUniqueIdentifier(), hashCode(), 1);\n";
			code += SalsaCompiler.getIndent() + "\treturn copy;\n";
			code += SalsaCompiler.getIndent() + "}\n";
		}

		if (!isAbstract) {
			/********
				*       Create this actor then send a message which will invoke construct on the actor.
			 ********/

			code += SalsaCompiler.getIndent() + "public static " + actorName + " construct(ActorReference creator, UAN uan, String host, int port, Object[] arguments) {\n";
			code += SalsaCompiler.getIndent() + "\tif (host == null) {\n";
			code += SalsaCompiler.getIndent() + "\t\thost = ServiceFactory.getReception().getLocalHost();\n";
			code += SalsaCompiler.getIndent() + "\t\tport = ServiceFactory.getReception().getLocalPort();\n";
			code += SalsaCompiler.getIndent() + "\t}\n\n";
			code += SalsaCompiler.getIndent() + "\tActor actor;\n";
			code += SalsaCompiler.getIndent() + "\tif (uan == null) {\n";
			code += SalsaCompiler.getIndent() + "\t\tactor = new " + actorName + "State(IdentifierGenerator.generateUniqueUAL(host, port));\n";
			code += SalsaCompiler.getIndent() + "\t} else {\n";
			code += SalsaCompiler.getIndent() + "\t\tactor = new " + actorName + "State(uan);\n";
			code += SalsaCompiler.getIndent() + "\t}\n";
			code += SalsaCompiler.getIndent() + "\t" + actorName +" target;\n";
			code += SalsaCompiler.getIndent() + "\tif (actor.getUAN() != null) target = new " + actorName + "(actor.getUAN());\n";
			code += SalsaCompiler.getIndent() + "\telse target = new " + actorName + "(actor.getUAL());\n";
			code += SalsaCompiler.getIndent() + "\tactor.setConstructMessage(new Message(creator, target, \"construct\", arguments, null, null, null));\n";
			code += SalsaCompiler.getIndent() + "\tServiceFactory.getStage().createActor(actor, host, port);\n";
			code += SalsaCompiler.getIndent() + "\treturn target;\n";
			code += SalsaCompiler.getIndent() + "}\n\n";

			/********
				*       Create this actor then send a message which will invoke act on the actor.
			 ********/
			code += SalsaCompiler.getIndent() + "public static void main(String[] arguments) {\n";
			code += SalsaCompiler.getIndent() + "\tServiceFactory.initialize();\n\n";
			code += SalsaCompiler.getIndent() + "\tUAN uan = null;\n";
			code += SalsaCompiler.getIndent() + "\tif (System.getProperty(\"actor_uan\") != null) {\n";
			code += SalsaCompiler.getIndent() + "\t\ttry {\n";
			code += SalsaCompiler.getIndent() + "\t\t\tuan = new UAN(System.getProperty(\"actor_uan\"));\n";
			code += SalsaCompiler.getIndent() + "\t\t} catch (MalformedUANException e) {\n";
			code += SalsaCompiler.getIndent() + "\t\t\tSystem.err.println(\"Error starting actor, malformed uan given: \" + System.getProperty(\"actor_uan\"));\n";
			code += SalsaCompiler.getIndent() + "\t\t\tSystem.err.println(\"\\tException: \" + e);\n";
			code += SalsaCompiler.getIndent() + "\t\t\treturn;\n";
			code += SalsaCompiler.getIndent() + "\t\t}\n";
			code += SalsaCompiler.getIndent() + "\t}\n";
			code += SalsaCompiler.getIndent() + "\tString host = System.getProperty(\"actor_host\");\n";
			code += SalsaCompiler.getIndent() + "\tint port = -1;\n";
			code += SalsaCompiler.getIndent() + "\tif (System.getProperty(\"actor_port\") != null) port = Integer.parseInt(System.getProperty(\"actor_port\"));\n";
			code += SalsaCompiler.getIndent() + "\tif (host == null) {\n";
			code += SalsaCompiler.getIndent() + "\t\thost = ServiceFactory.getReception().getLocalHost();\n";
			code += SalsaCompiler.getIndent() + "\t\tport = ServiceFactory.getReception().getLocalPort();\n";
			code += SalsaCompiler.getIndent() + "\t}\n\n";
			code += SalsaCompiler.getIndent() + "\tActor actor;\n";
			code += SalsaCompiler.getIndent() + "\t" + actorName + " self;\n";
			code += SalsaCompiler.getIndent() + "\tif (uan == null) {\n";
			code += SalsaCompiler.getIndent() + "\t\tactor = new " + actorName + "State(IdentifierGenerator.generateUniqueUAL(host, port));\n";
//			if (System.getProperty("enable_gc") == null) {
				code += SalsaCompiler.getIndent() + "\t\tself = new " + actorName + "(actor.getUAL());\n";
//			} else {
//				code += SalsaCompiler.getIndent() + "\t\tself = new " + actorName + "(actor.getUAL()).copy();\n";
//			}
			code += SalsaCompiler.getIndent() + "\t} else {\n";
			code += SalsaCompiler.getIndent() + "\t\tactor = new " + actorName + "State(uan);\n";

//			if (System.getProperty("enable_gc") == null) {
				code += SalsaCompiler.getIndent() + "\t\tself = new " + actorName + "(actor.getUAN());\n";
//			} else {
//				code += SalsaCompiler.getIndent() + "\t\tself = new " + actorName + "(actor.getUAN()).copy();\n";
//			}
			code += SalsaCompiler.getIndent() + "\t}\n";
			code += SalsaCompiler.getIndent() + "\tactor.setConstructMessage(new Message(self, self, \"act\", new Object[]{ arguments }, null, null, null));\n";
			code += SalsaCompiler.getIndent() + "\tServiceFactory.getStage().createActor(actor, host, port);\n";
			code += SalsaCompiler.getIndent() + "}\n";
                }
		SalsaCompiler.indent = 0;
		code += "}\n";
		return code;
	}

	public String getStateCode() {
		return getJavaCode();
	}


	public String getChildCode() {
		int place = 0;

		String code = "";
		if (getChild(0) instanceof ModuleDeclaration) {
			code += getChild(0).getJavaCode();
			place++;
		}

		code += "// Import declarations generated by the SALSA compiler, do not modify.\n";
		code += "import java.lang.reflect.InvocationTargetException;\n";
		code += "import java.lang.reflect.Method;\n";

		code += "import salsa_lite.core.language.Actor;\n";
		code += "import salsa_lite.core.language.ActorReference;\n";
		code += "import salsa_lite.core.language.IdentifierGenerator;\n";
		code += "import salsa_lite.core.language.Message;\n\n";

		code += "import salsa_lite.core.services.ServiceFactory;\n\n";

		if (getChild(children.length-1) instanceof BehaviorDeclaration) {
			code += "import salsa_lite.core.language.Token;\n";
			code += "import salsa_lite.core.language.exceptions.CurrentContinuationException;\n";
			code += "import salsa_lite.core.language.exceptions.MessageHandlerNotFoundException;\n";

			String actorType = System.getProperty("actor");
			if (actorType == null) {
				if (System.getProperty("enable_gc") == null) {
					actorType = "salsa_lite.core.language.LiteActorState";
				} else {
					actorType = "salsa_lite.core.language.CollectibleActorState";
				}
			} else {
				actorType += "State";
			}

			code += "\n";
			code += "import " + actorType + ";\n\n";
			code += "import salsa_lite.core.naming.UAN;\n";
			code += "import salsa_lite.core.naming.UAL;\n";
			code += "import salsa_lite.core.naming.MalformedUALException;\n";
			code += "import salsa_lite.core.naming.MalformedUANException;\n\n";
		}

		code += "// End SALSA compiler generated import delcarations.\n\n";

		for (int i = place; i < children.length; i++) {
			if (getChild(i) instanceof BehaviorDeclarationAttributes) code += "\n";
			code += getChild(i).getJavaCode();
		}

		SalsaCompiler.indent = 0;
		return code;
	}
}
