package salsa_lite.core.language;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.util.Vector;
import java.util.Hashtable;

import salsa_lite.core.language.exceptions.CurrentContinuationException;
import salsa_lite.core.naming.UAN;
import salsa_lite.core.naming.UAL;
import salsa_lite.core.naming.URI;
import salsa_lite.core.services.ServiceFactory;

public class CollectibleActor implements ActorReference, java.io.Serializable {
	private UAN uan;
	private UAL ual;
	private int hashcode;
	public boolean selfReference = true;

	public UAN getUAN()			{ return uan; }
	public UAL getUAL()			{ return ual; }
	public URI getURI() {
		if (uan != null) return uan;
		else return ual;
	}

	public int hashCode()			{ return hashcode; }

	public CollectibleActor(UAN uan) {
		this.uan = uan;
		hashcode = getUniqueIdentifier().hashCode();
	}
	public CollectibleActor(UAL ual) {
		this.ual = ual;
		hashcode = getUniqueIdentifier().hashCode();
	}

	public CollectibleActor copy() {
		CollectibleActor copy = null;
		if (uan != null) {
			copy = new CollectibleActor(uan);
		} else {
			copy = new CollectibleActor(ual);
		}
		copy.selfReference = false;
		ServiceFactory.getGarbage().incrementReference(getUniqueIdentifier(), hashcode, 1);
		return copy;
	}

	protected void finalize() {
		if (!selfReference) {
			ServiceFactory.getGarbage().decrementReference(getUniqueIdentifier(), hashcode, 1);
		}
	}

	public String getUniqueIdentifier() {
		if (uan != null) {
			return uan.toString();
		} else if (ual != null) {
			return ual.toString();
		} else {
			System.err.println("ActorReference error:");
			System.err.println("\tUnidentified Actor Reference: " + this);
			return null;
		}
	}

	public boolean equals(Object o) {
		if (o instanceof Actor) {
			Actor other = (Actor)o;
			return getUniqueIdentifier().equals(other.getUniqueIdentifier());
		} else if (o instanceof ActorReference) {
			ActorReference other = (ActorReference)o;
			return getUniqueIdentifier().equals(other.getUniqueIdentifier());
		}
		return false;
	}

	public String toString() {
		return "[" + getClass().getName() + ", " + getUniqueIdentifier() + ", self? " + selfReference + "]";
	}
}
