package salsa_lite.local;

import salsa_lite.core.services.StageService;
import salsa_lite.core.services.ServiceFactory;

import salsa_lite.core.language.Actor;
import salsa_lite.core.language.Message;

import salsa_lite.core.naming.UAL;

import java.util.Hashtable;
import java.util.Vector;

public class LocalStageService implements StageService {
	int number_stages = 1;
	StageActor[] stages;

	public LocalStageService() {
		System.err.println("created stage service");

		if (System.getProperty("nstages") != null) {
			number_stages = Integer.parseInt( System.getProperty("nstages") );
		}

		stages = new StageActor[number_stages];
		for (int i = 0; i < number_stages; i++) {
			stages[i] = new StageActor(i);
			stages[i].start();
			System.err.println("created stage[" + i + "]");
		}
	}


	/********
		*	Direct messages and actors to the appropriate stage, based on a hash
		*	function.
	 ********/
	public void createActor(Actor actor, String host, int port) {
		int target = getTarget(actor.hashCode());
		stages[target].putActorOnStage(actor);
	}

	public void sendMessageFromLocal(Message message) {
		int target = getTarget(message.getTarget().hashCode());
		stages[target].putMessageInMailbox(message);
	}


	/********
		*	Hash function to determine which stage an actor is on
	 ********/
	public int getTarget(int hashcode) {
		return Math.abs(hashcode % number_stages);
	}


	/********
		*	Methods for remote message sending and migration should not be used.
	 ********/
	public void actorMigration(Actor actor, String remoteHost, int remotePort) {
		System.err.println("LocalLookupService error, cannot call actorMigration");
		System.err.println("\tactor: " + actor);
	}

	public void sendMessageFromRemote(Message message, String remoteHost, int remotePort) {
		System.err.println("LocalLookupService error, cannot call sendMessageFromRemote");
		System.err.println("\tmessage: " + message);
		System.err.println("\ttarget: " + remoteHost + ":" + remotePort);
	}

	public void handleObjectFromRemote(Object object, String remoteHost, int remotePort) {
		System.err.println("LocalLookupService error, cannot call handleObjectFromRemote");
		System.err.println("\tobject: " + object);
		System.err.println("\ttarget: " + remoteHost + ":" + remotePort);
	}
}
