package salsa_lite.local_gc;

import java.util.Hashtable;
import java.util.Vector;

import salsa_lite.core.language.ActorReference;
import salsa_lite.core.language.Message;
import salsa_lite.core.services.StageService;
import salsa_lite.core.services.ServiceFactory;

public class CollectionActor {

	private Hashtable<String,Long> weights;
	private Hashtable<String,ActorReference> references;
	private Vector<Message> mailbox;
	private int id;
	private StageService stageService;
	
	public CollectionActor(int id) {
		this.id = id;
		weights = new Hashtable<String,Long>();
		references = new Hashtable<String,ActorReference>();
		mailbox = new Vector<Message>();
		stageService = ServiceFactory.getStage();
	}

	public void register(String id, ActorReference reference) {
		references.put(id, reference);
		weights.put(id, new Long(0));
//		System.err.println("REGISTERED: " + id);
	}

	public void increment(String id, long weight) {
		long current = weights.get(id) + weight;
		weights.put(id, current);
//		System.err.println("INCREMENTED: " + id + " by " + weight + " to " + current);
	}

	public void decrement(String id, long weight) {
		long current = weights.get(id) - weight;
		if (current == 0) {
			weights.remove(id);
			ActorReference reference = references.remove(id);
			Message destroyMessage = new Message(null, reference, "destroy", new Object[]{}, null, null);
			stageService.sendMessageFromLocal(destroyMessage);
//			System.err.println("DECREMENTED: " + id + " by " + weight + " to " + current);
//			System.err.println("DESTROYING: " + id + ", left: " + references.size() + ", weights: " + weights.size());
		} else {
			weights.put(id, current);
//			System.err.println("DECREMENTED: " + id + " by " + weight + " to " + current);
		}
	}
}
