package salsa_lite.wwc.stage;

import salsa_lite.core.language.Actor;
import salsa_lite.core.language.ActorReference;
import salsa_lite.core.language.Message;
import salsa_lite.core.naming.UAL;
import salsa_lite.core.services.StageService;

public class WWCStageService implements StageService {

	int number_actors = 1;
	StageActor[] stages;

	public WWCStageService() {
		if (System.getProperty("nstages") != null) {
			number_actors = Integer.parseInt( System.getProperty("nstages") );
		}

		stages = new StageActor[number_actors];
		for (int i = 0; i < number_actors; i++) {
			stages[i] = new StageActor(i);
			System.err.println("Created Stage[" + i + "]");
			stages[i].start();
		}
	}

	private int getTarget(int hashcode) {
		return Math.abs(hashcode % number_actors);
	}

	public void actorMigration(Actor actor, String remoteHost, int remotePort) {
		int target = getTarget(actor.hashCode());
		stages[target].put( new ActorMigrationMessage(actor, remoteHost, remotePort) );
	}

	public void createActor(Actor actor, String targetHost, int targetPort) {
		int target = getTarget(actor.hashCode());
		stages[target].put( new CreateActorMessage(actor, targetHost, targetPort) );
	}

	public void sendMessageFromRemote(Message message, String remoteHost, int remotePort) {
		int target = getTarget(message.getTarget().hashCode());
		stages[target].put( new SendFromRemoteMessage(message, remoteHost, remotePort) );
	}

	public void sendMessageFromLocal(Message message) {
		int target = getTarget(message.getTarget().hashCode());
		stages[target].put( new SendFromLocalMessage(message) );
	}

	public void handleObjectFromRemote(Object object, String remoteHost, int remotePort) {
		if (object instanceof UpdateInvalidReferenceMessage || object instanceof UpdatedInvalidReferenceMessage) {
			int target = getTarget( object.hashCode() );
			stages[target].put( object );
		} else {
			System.err.println("Error: StageService received unknown object: " + object);
			System.err.println("\tfrom: " + remoteHost + ":" + remotePort);
		}
	}
}
