package salsa_lite.wwc.transport;

import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.net.Socket;
import java.net.SocketException;
import java.net.ConnectException;
import java.net.InetSocketAddress;
import java.util.Vector;

import salsa_lite.core.services.ServiceFactory;

public class OutgoingSocketHandler extends Thread {
	private ObjectOutputStream outputStream;
	Vector<Object> objects;

	String targetHost;
	int targetPort;

	protected OutgoingSocketHandler(String targetHost, int targetPort) {
		this.targetHost = targetHost;
		this.targetPort = targetPort;
		objects = new Vector<Object>();

		start();
	}

	private void initialize() {
		try {
			outputStream = new ObjectOutputStream(new Socket(targetHost, targetPort).getOutputStream());
		} catch (Exception e) {
			System.err.println("WWC Transport Service Error: ");
			System.err.println("\tThe target host '" + targetHost + ":" + targetPort + "' is not available");
			System.err.println("\t" + e);
		}
		try {
			outputStream.writeObject( ServiceFactory.getReception().getLocalHost() );
			outputStream.writeObject( ServiceFactory.getReception().getLocalPort() );
			outputStream.flush();
		} catch (Exception e) {
			System.err.println("WWC Transport Service Error: ");
			System.err.println("\tCould not transfer local host and port to '" + targetHost + ":" + targetPort + "'");
			System.err.println("\t" + e);
		}
	}

	protected synchronized void send(Object object) {
		objects.add(object);
		notify();
	}

	protected synchronized Object getNext() {
		if (objects.isEmpty()) {
			try {
				wait();
			} catch (Exception e) {
				System.err.println("Error, OutgoingSocketHandler wait.");
				System.err.println("Exception: e");
				e.printStackTrace();
				return null;
			}
		}
		return objects.remove(0);
	}

	int errors = 0;
	public void run() {
		initialize();

		while (true) {
			Object toSend = getNext();
			try {
				outputStream.writeObject(toSend);
				outputStream.flush();
			} catch (Exception e) {
				System.err.println("Error in OutgoingSocketHandler, writing object: " + toSend);
				System.err.println("Exception: " + e);
				e.printStackTrace();
				return;
			}
		}
	}
}
